import sympy as sy
from . import configuration
from . import core


def __check_properties__(instance, property):
    if hasattr(instance.properties, property):
        out = instance.properties[property]
    else:
        out = None


def name_variable_generator(instance):
    if hasattr(instance, "name") and (instance.name != ""):
        instance_name_variable = instance.name
    else:
        if hasattr(instance, "name_expression"):
            instance_name_variable = instance.name_expression
        else:
            instance_name_variable = str(instance)
    return instance_name_variable


def name_variables_generator(first, second):
    first_name_variable = name_variable_generator(first)
    second_name_variable = name_variable_generator(second)
    return [first_name_variable, second_name_variable]


def numerical_variable_generator(instance):
    if hasattr(instance, "numerical"):
        instance_numerical = instance.numerical
    else:
        instance_numerical = instance
    return instance_numerical


def numerical_variables_generator(first, second):
    first_numerical = numerical_variable_generator(first)
    second_numerical = numerical_variable_generator(second)
    return [first_numerical, second_numerical]


def symbolic_expression_variable_generator(instance):
    if hasattr(instance, "symbol"):
        if instance.__symbol__ == configuration.undefined_unit_symbol:
            instance_symbolic_variable = instance.symbolic_expression
        else:
            instance_symbolic_variable = instance.symbol
    else:
        instance_symbolic_variable = sy.Symbol(configuration.undefined_unit_symbol)
    return instance_symbolic_variable


def symbolic_expression_variables_generator(first, second):
    first_symbolic_variable = symbolic_expression_variable_generator(first)
    second_symbolic_variable = symbolic_expression_variable_generator(second)
    return [first_symbolic_variable, second_symbolic_variable]


def unit_variable_generator(instance):
    if hasattr(instance, "unit"):
        return instance.unit
    else:
        return core.Unit()


def full_variable_generator(instance):
    name_variable = name_variable_generator(instance)
    symbolic_variable = symbolic_expression_variable_generator(instance)
    numerical_variable = numerical_variable_generator(instance)
    unit_variable = unit_variable_generator(instance)
    return {
        "name": name_variable,
        "symbolic": symbolic_variable,
        "numerical": numerical_variable,
        "unit": unit_variable,
    }



import numpy as np
import sympy as sy
import sympy.functions.special.elliptic_integrals
import scipy as sp
import scipy.special
from . import core
from . import helpers


def variable_compatibility_check(input):
    if hasattr(input, "__parent_class__"):
        if input.__parent_class__ == "Value":
            new = input.__class__()
        else:
            new = core.Variable()
    else:
        new = core.Variable()
    return new


def complete_elliptical_integral_first_kind(instance=core.Variable()):
    new = variable_compatibility_check(instance)
    instance_parameters = helpers.full_variable_generator(instance)
    new.numerical = sp.special.ellipk(instance_parameters["numerical"])
    new.symbolic_expression = sy.functions.special.elliptic_integrals.elliptic_k(
        instance_parameters["symbolic"]
    )
    new.name_expression = (
        "complete_elliptical_integral_first_kind(" + instance_parameters["name"] + ")"
    )
    # TODO automate this
    new.unit.name = "K(" + instance_parameters["unit"].name + ")"
    # TODO get symbolic parameters
    new.unit.symbolic_expression = sy.functions.special.elliptic_integrals.elliptic_k(
        instance_parameters["unit"].symbolic_expression
    )
    return new


def sqrt(instance=core.Variable()):
    new = variable_compatibility_check(instance)
    instance_parameters = helpers.full_variable_generator(instance)
    new.numerical = np.sqrt(instance_parameters["numerical"])
    new.symbolic_expression = sy.sqrt(instance_parameters["symbolic"])
    new.name_expression = "square_root(" + instance_parameters["name"] + ")"
    # TODO automate this
    new.unit.name = "square_root(" + instance_parameters["unit"].name + ")"
    # TODO get symbolic parameters
    new.unit.symbolic_expression = sy.sqrt(
        instance_parameters["unit"].symbolic_expression
    )
    return new


def sinh(instance=core.Variable()):
    new = variable_compatibility_check(instance)
    instance_parameters = helpers.full_variable_generator(instance)
    new.numerical = np.sinh(instance_parameters["numerical"])
    # TODO operate on both symbolic variables
    new.symbolic_expression = sy.sinh(instance_parameters["symbolic"])
    new.name_expression = "sinh(" + instance_parameters["name"] + ")"
    return new


# def ellipk(instance=core.Variable()):
#    sp.special.ellipk(m)


# if __name__ == "__main__":
#     sinh()



#!/usr/bin/env python
import setuptools

with open("README_RAW.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="numpsy",
    version="0.0.4",
    description="NumpSy - Integrated NumPy, SymPy, SciPy and Pandas with unit management for scientific programming with IPython.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/daquintero/numpsy",
    author="Dario Quintero",
    author_email="darioaquintero@gmail.com",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
)
